/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>
#import "Sparrow.h" 

@class ActorType;
@class Behavior;
@class BehaviorManager;
@class Collision;
@class Sprite;

struct b2BodyDef;
#ifndef B2BODYDEF_DEFINED
typedef struct b2BodyDef b2BodyDef;
# define B2BODYDEF_DEFINED
#endif

struct b2MassData;
typedef struct b2MassData b2MassData;

struct b2Vec2;
typedef struct b2Vec2 b2Vec2;


//----------------------------------------------------------------------------------
// Interface
//----------------------------------------------------------------------------------

@interface Actor : SPSprite
{	
	@private
        //A globalID for a run through a scene. Used to workaround bug where
        //tweened actors at scene switch time can persist to next scene due to
        //tween not releasing in time and then removing from previous b2World.
        int mWorldID;
    
		BOOL mRecycled;
        BOOL mPaused;
    
		NSMutableDictionary* mRegistry;
		BOOL mHandlesCollisions;
		Actor* mLastCollided;
		NSMutableDictionary* mContacts;
		NSMutableDictionary* mCollisions;
		
		int mID;
		//Name is part of superclass
		
		int mLayerID;
		int mGroupID;
		int mTypeID;
    
        NSNumber* mGroupIDWrapper;
        NSNumber* mTypeIDWrapper;
		
		Sprite* mSprite;
		NSMutableDictionary* mShapeMap;
        NSMutableDictionary* mOriginMap;
		
		NSMutableDictionary* mAnims;
		NSString* mDefaultAnim;
		NSString* mCurrAnim;
		
		SPMovieClip *mCurrentAnimation; //NEW
        float mAnimWidth;
        float mAnimHeight;
    
		BOOL mIsRegion;
		BOOL mIsTerrainRegion;
		
		//game already handled
		
		#ifdef __cplusplus
			b2Body* mBody;
			b2BodyDef* mBodyDef;
			b2MassData* mMassData; //md in Flash
            b2MouseJoint* mMouseJoint;
    
            b2Vec2* mCurrOrigin;
            b2Vec2* mCurrOffset;
    
            b2WorldManifold* mManifold;
		#endif
		
		BehaviorManager *mBehaviors; //manager in Flash
		
		BOOL mDestroyed;
		BOOL mDrawActor;
		BOOL mIsHUD;
		BOOL mAlwaysSimulate;
        BOOL mDraggable; //Automagically enable mouse-joint powered dragging
        int mDragStrength;
    
		float mTweenLocX;
		float mTweenLocY;
		float mTweenAngle;
        float mTweenScaleX;
        float mTweenScaleY;
        float mTweenFade;
	
		SPTween* mLocTween;
		SPTween* mAngleTween;
		SPTween* mScaleTween;
        SPTween* mFadeTween;
    
		BOOL mIsCamera;
		BOOL mKillLeaveScreen;
	
		//From FlxSprite
		BOOL mExists;
		BOOL mDead;
    
        //Optional Label
        SPTextField* mLabel;
    
        //No Box2D
        BOOL mIsLightweight;
        float mXVel;
        float mYVel;
        float mAVel;
    
        BOOL mIsDown;
        BOOL mOnStarted;
        BOOL mOnDragged;
        BOOL mOnReleased;
    
        //Memory Related
        BOOL mDeleteBodyDef;
    
        //Events
        NSMutableDictionary* mAllListeners;
        NSMutableArray* mAllListenerReferences;
        
        NSMutableArray* mWhenCreatedListeners;
        NSMutableArray* mWhenUpdatedListeners;
        NSMutableArray* mWhenDrawingListeners;
        NSMutableArray* mWhenKilledListeners;
        
        NSMutableArray* mTouchListeners;
        NSMutableArray* mPositionListeners;
        NSMutableArray* mCollisionListeners;
    
        BOOL mLastScreenState;
        BOOL mLastSceneState;
}

#ifdef __cplusplus
+ (Actor*)init:(Actor*)a ID:(int)ID groupID:(int)groupID x:(float)x y:(float)y layerID:(int)layerID width:(int)width height:(int)height sprite:(Sprite*)sprite behaviorValues:(NSMutableDictionary*)behaviorValues actorType:(ActorType*)actorType bodyDef:(b2BodyDef*)bodyDef isSensor:(BOOL)isSensor isStationary:(BOOL)isStationary isKinematic:(BOOL)isKinematic canRotate:(BOOL)canRotate shape:(b2Shape*)shape invisible:(BOOL)invisible typeID:(int)typeID lightweight:(BOOL)lightweight;
#endif

- (void)stripBehaviors;

//Initialization
- (void)initScripts;

#ifdef __cplusplus
+ (b2Shape*)createBox:(int)width height:(int)height;
#endif

- (void)initFromBody:(b2BodyDef*)bodyDef;

#ifdef __cplusplus
- (void)initBody:(int)groupID isSensor:(BOOL)isSensor isStationary:(BOOL)isStationary isKinematic:(BOOL)isKinematic canRotate:(BOOL)canRotate shape:(b2Shape*)shape;
#endif

//Animation
- (void)addAnim:(NSString *)animationName anim:(NSMutableArray*)a frameCount:(int)frameCount frameWidth:(int)frameWidth frameHeight:(int)frameHeight originX:(float)originX originY:(float)originY durations:(NSMutableArray*)durations looping:(BOOL)looping shapes:(NSMutableDictionary*)shapes; //=addAnim
- (NSString*)getAnimation;
- (void)setAnimation:(NSString*)name;
- (void)switchToDefaultAnimation;
- (BOOL)isAnimationPlaying;
- (int)getCurrentFrame;
- (void)setCurrentFrame:(int)frameIndex;
- (int)getFrameCount; //=getNumFrames
- (void)switchAnimationTo:(NSString *)animationName; //=switchAnimation
- (void)scaleBody;

//Events
- (void)registerCollisionHandler:(Behavior*)b;
- (void)update;
- (void)innerUpdate:(BOOL)hudCheck;
- (void)internalUpdate:(BOOL)doAll;
- (void)handleCollisions;
#ifdef __cplusplus
- (BOOL)collidedFromBottom:(b2Contact*)c normal:(b2Vec2*)normal;
- (BOOL)collidedFromTop:(b2Contact*)c normal:(b2Vec2*)normal;
- (BOOL)collidedFromLeft:(b2Contact*)c normal:(b2Vec2*)normal;
- (BOOL)collidedFromRight:(b2Contact*)c normal:(b2Vec2*)normal;
#endif
- (BOOL)collidedWithActor:(Actor*)a;
- (BOOL)collidedWithTile:(Actor*)a;
#ifdef __cplusplus
- (void)addContact:(b2Contact*)point;
- (void)removeContact:(b2Contact*)point;
#endif
- (void)handleCollision:(Collision*)event;
- (void)updateAnimProperties:(BOOL)doAll;
- (void)drawImage:(SPRenderSupport*)g;
- (void)render:(SPRenderSupport*)support; // = renderSprite
- (SPTexture*)getImage;

//ID, name, groupID, layerID are properties
- (int)getLayerOrder;
- (ActorType*)getType;
- (void)pause;
- (void)unpause;
- (BOOL)isPaused;

//Layering
- (void)moveToLayerOrder:(int)layerOrder;
- (void)bringToFront;
- (void)bringForward;
- (void)sendToBack;
- (void)sendBackward;

//Physics: Position
- (float)getX;
- (float)getY;
- (float)getXCenter;
- (float)getYCenter;
- (float)getXOnScreen; //getScreenX
- (float)getYOnScreen; //getScreenY

- (void)setXPosition:(float)x;
- (void)setYPosition:(float)y;
- (void)followActor:(Actor*)actor;
- (void)followActor:(Actor*)actor withOffsetX:(int)offsetX withOffsetY:(int)offsetY;

//Physics: Angles
- (float)getAngle;
- (float)getAngleInDegrees;
- (void)setAngle:(float)angle;
- (void)setAngleInDegrees:(float)angleInDegrees;
- (void)rotate:(float)angle;
- (void)rotateDegrees:(float)angleInDegrees;
- (float)getAngularVelocity;
- (void)setAngularVelocity:(float)angularVelocity;
- (void)changeAngularVelocityBy:(float)angularVelocity;

//Physics: Velocity
- (float)getXVelocity;
- (float)getYVelocity;
- (void)setXVelocity:(float)dx;
- (void)setYVelocity:(float)dy;
- (void)setVelocityAtAngle:(float)angle withSpeed:(float)speed;
- (void)accelerateX:(float)dx;
- (void)accelerateY:(float)dy;
- (void)accelerateAtAngle:(float)angle withSpeed:(float)speed;

//Physics: Forces
- (void)push:(float)dirX dirY:(float)dirY withForce:(float)magnitude;
- (void)pushInDirection:(float)angle withForce:(float)magnitude;
- (void)applyImpulse:(float)dirX dirY:(float)dirY withForce:(float)magnitude;
- (void)applyImpulseInDirection:(float)angle withForce:(float)magnitude;
- (void)applyTorque:(float)torque;

//Physics: Size
- (float)getWidth;
- (float)getHeight;
- (float)getPhysicalWidth;
- (float)getPhysicalHeight;

//Physics: General
#ifdef __cplusplus
- (b2Body*)getBody;
#endif
- (void)enableRotation;
- (void)disableRotation;
- (BOOL)canRotate;

- (void)setIgnoreGravity:(BOOL)flag;
- (BOOL)ignoresGravity;

//Touched Actor
- (void)processTouch:(SPTouch*)touch;
- (BOOL)touchedActor;
- (BOOL)touchingActor;
- (BOOL)draggedActor;
- (BOOL)releasedActor;
- (void)makeDraggableWithSensitivity:(int)amount;

//Tweening
- (void)cancelTweens;
- (void)moveToX:(float)x y:(float)y time:(float)time transition:(NSString*)transition;
- (void)moveByX:(float)x y:(float)y time:(float)time transition:(NSString*)transition;
- (void)spinTo:(float)angle time:(float)time transition:(NSString*)transition;
- (void)spinBy:(float)angle time:(float)time transition:(NSString*)transition;
- (void)scaleToWidth:(float)width height:(float)height time:(float)time transition:(NSString*)transition;
- (void)fadeTo:(float)alpha time:(float)time transition:(NSString*)transition;

//Draw Sprite
- (void)enableActorDrawing;
- (void)disableActorDrawing;
- (BOOL)drawsImage;
//setFilter

//Behaviors
- (void)addBehavior:(Behavior*)b;
- (BOOL)hasBehavior:(NSString*)name;
- (void)enableBehavior:(NSString*)name;
- (void)disableBehavior:(NSString*)name;
- (BOOL)isBehaviorEnabled:(NSString*)name;

//Messaging
- (id)getAttributeForBehavior:(NSString*)behaviorName att:(NSString*)attName; //getValue
- (void)setAttributeForBehavior:(NSString*)behaviorName att:(NSString*)attName withValue:(void*)value; //setValue
- (void)shout:(NSString*)msg;
- (id)sayToBehavior:(NSString*)recipient msg:(NSString*)msg withArgs:(NSArray*)args;

//Actor-Level Attributes
- (void)setActorValue:(NSString*)name value:(id)value;
- (id)getActorValue:(NSString*)name;
- (BOOL)hasActorValue:(NSString*)name;

//Misc (HUD, IST)
- (void)anchorToScreen;
- (void)unanchorFromScreen;
- (BOOL)isAnchoredToScreen;
- (void)makeAlwaysSimulate;
- (BOOL)alwaysSimulates;
- (void)die;
- (void)killSelfAfterLeavingScreen;
- (BOOL)isDying;
+ (BOOL)isAlive:(Actor*)actor;
- (BOOL)isOnScreen;
- (Actor*)getLastCollidedActor;

//Camera
- (void)setLocation:(int)x y:(int)y;
- (void)killSelfAterLeavingScreen;

//Misc
-(NSString*)toString;

//Labels
- (void)attachLabel;
- (void)destroyLabel;
- (void)setLabelText:(NSString*)text;
- (void)setLabelFont:(NSString*)font;

- (void)setLabelToLeft;
- (void)setLabelToCenter;
- (void)setLabelToRight;
- (void)setLabelWidth:(int)width;

//Events
- (void)registerListener:(NSMutableArray*)type listener:(id)listener;
- (void)removeAllListeners;

- (void)checkScreenState;


//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) BOOL recycled;
@property (nonatomic, assign) BOOL paused;
@property (nonatomic, assign) NSMutableDictionary* registry;
@property (nonatomic, assign) BOOL handlesCollisions;
@property (nonatomic, assign) Actor* lastCollided;
@property (nonatomic, assign) NSMutableDictionary* contacts;
@property (nonatomic, assign) NSMutableDictionary* collisions;

@property (nonatomic, assign) int ID;
@property (nonatomic, assign) int worldID;
@property (nonatomic, assign) int layerID;
@property (nonatomic, assign) int groupID;
@property (nonatomic, assign) int typeID;
@property (nonatomic, assign) NSNumber* groupIDWrapper;
@property (nonatomic, assign) NSNumber* typeIDWrapper;

@property (nonatomic, assign) Sprite* sprite;
@property (nonatomic, assign) NSMutableDictionary* shapeMap;
@property (nonatomic, assign) NSMutableDictionary* originMap;

@property (nonatomic, assign) NSMutableDictionary* anims;
@property (nonatomic, assign) NSString* defaultAnim;
@property (nonatomic, assign) NSString* currAnim;

@property (nonatomic, assign) SPMovieClip* currentAnimation;
@property (nonatomic, assign) float animWidth;
@property (nonatomic, assign) float animHeight;


@property (nonatomic, assign) BOOL isRegion;
@property (nonatomic, assign) BOOL isTerrainRegion;

#ifdef __cplusplus
@property (nonatomic, assign) b2Body* body;
@property (nonatomic, assign) b2BodyDef* bodyDef;
@property (nonatomic, assign) b2MassData* massData;

@property (nonatomic, assign) b2Vec2* currOrigin;
@property (nonatomic, assign) b2Vec2* currOffset;
#endif

@property (nonatomic, assign) BehaviorManager* behaviors;

@property (nonatomic, assign) BOOL destroyed;
@property (nonatomic, assign) BOOL drawActor;
@property (nonatomic, assign) BOOL isHUD;
@property (nonatomic, assign) BOOL alwaysSimulate;
@property (nonatomic, assign) BOOL draggable;
@property (nonatomic, assign) int dragStrength;

@property (nonatomic, assign) float tweenLocX;
@property (nonatomic, assign) float tweenLocY;
@property (nonatomic, assign) float tweenAngle;
@property (nonatomic, assign) float tweenScaleX;
@property (nonatomic, assign) float tweenScaleY;
@property (nonatomic, assign) float tweenFade;

@property (nonatomic, assign) BOOL isCamera;
@property (nonatomic, assign) BOOL killLeaveScreen;

//FlxSprite
@property (nonatomic, assign) BOOL exists;
@property (nonatomic, assign) BOOL dead;

@property (nonatomic, assign) BOOL lightweight;
@property (nonatomic, assign) float xVel;
@property (nonatomic, assign) float yVel;
@property (nonatomic, assign) float aVel;

//Events
@property (nonatomic, assign) NSMutableDictionary* allListeners;
@property (nonatomic, assign) NSMutableArray* allListenerReferences;

@property (nonatomic, assign) NSMutableArray* whenCreatedListeners;
@property (nonatomic, assign) NSMutableArray* whenUpdatedListeners;
@property (nonatomic, assign) NSMutableArray* whenDrawingListeners;
@property (nonatomic, assign) NSMutableArray* whenKilledListeners;

@property (nonatomic, assign) NSMutableArray* touchListeners;
@property (nonatomic, assign) NSMutableArray* positionListeners;
@property (nonatomic, assign) NSMutableArray* collisionListeners;

@property (nonatomic, assign) BOOL lastScreenState;
@property (nonatomic, assign) BOOL lastSceneState;

@end
