/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>
#import "Sparrow.h"
#import "TBXML.h"

@class Resource;

@interface Assets : NSObject 
{
	@private
		NSMutableArray* readers;
	
		TBXMLElement* mGameXML;	
		TBXMLElement* mResourceListXML;
		TBXMLElement* mSceneListXML;
		TBXMLElement* mBehaviorListXML;
	
		//Map of each [sceneID].xml by ID
		NSMutableDictionary* mScenesXML;
		
		//Map of each [sceneID].scn by ID
		NSMutableDictionary* mScenesTerrain;
		
		//Map of each resource in memory by ID
		NSMutableDictionary* mResources;
		
		//Map of each static asset by filename
		NSMutableDictionary* mResourceAssets;
		
		//Map of each behavior by ID
		NSMutableDictionary* mBehaviors;
	
        NSMutableDictionary* mAtlases;
    
        NSMutableSet* mAtlasesToLoad;
        NSMutableSet* mAtlasesToUnload;
}

+(Assets*)get;

-(void)initAssets;
-(void)initMasterLists;
-(void)loadScenes;

-(void)loadReaders;
-(void)loadAtlases;
-(void)loadBehaviors;
-(void)loadResources;
-(void)readResourceXML:(TBXMLElement*)xml;
-(Resource*)readResource:(int)ID type:(NSString*)type name:(NSString*)name xml:(TBXMLElement*)xml;
-(TBXMLElement*)toXML:(NSString*)data;
-(NSMutableArray*)getResourcesOfType:(Class)type;

//Memory Management / Tuning
- (void)loadAtlas:(int)atlasID;
- (void)unloadAtlas:(int)atlasID;
- (void)loadAllAtlases;
- (void)updateAtlases;



//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) TBXMLElement* gameXML;
@property (nonatomic, assign) TBXMLElement* resourceListXML;
@property (nonatomic, assign) TBXMLElement* sceneListXML;
@property (nonatomic, assign) TBXMLElement* behaviorListXML;

@property (nonatomic, assign) NSMutableDictionary* scenesXML;
@property (nonatomic, assign) NSMutableDictionary* scenesTerrain;
@property (nonatomic, assign) NSMutableDictionary* resources;
@property (nonatomic, assign) NSMutableDictionary* resourceAssets;
@property (nonatomic, assign) NSMutableDictionary* behaviors;
@property (nonatomic, assign) NSMutableDictionary* atlases;

@end
