/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>

@class Behavior;
@class SPRenderSupport;

@interface BehaviorManager : NSObject 
{
	@private
		NSMutableArray *mBehaviors;
		NSMutableArray *mDrawBehaviors;
		NSMutableArray *mCollisionHandlers;
	
		NSMutableDictionary *mCache;
}

+ (BehaviorManager*)behaviorManager;

- (void)addBehavior:(Behavior*)behavior;
- (void)enableBehavior:(NSString*)behavior;
- (void)disableBehavior:(NSString*)behavior;
- (BOOL)hasBehavior:(NSString*)behavior;
- (BOOL)isBehaviorEnabled:(NSString*)behavior;

- (void)load;
- (void)initScripts;

- (void)update;
- (void)draw:(SPRenderSupport*)g x:(int)x y:(int)y screen:(BOOL)screen;
- (void)registerCollisionHandler:(Behavior*)behavior;

//Messaging
- (void)shout:(NSString*)msg;
- (id)say:(NSString*)msg to:(NSString*)recipient withArgs:(NSArray*)args;
- (void)setAttribute:(NSString*)attName from:(NSString*)behaviorName withValue:(void*)value;
- (id)getAttribute:(NSString*)attName from:(NSString*)behaviorName;


//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) NSMutableArray* behaviors;
@property (nonatomic, assign) NSMutableArray* drawBehaviors;
@property (nonatomic, assign) NSMutableArray* collisionHandlers;

@end
