/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>
#import <OpenGLES/EAGL.h>
#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>
#import <GameKit/GameKit.h>
#import "Sparrow.h" 
#import "ParallaxArea.h"
#include "SHSwipeEvent.h"
#include "IAPHelper.h"
#include "Script.h"

#define STEP_SIZE 0.01
#define PHYSICS_SCALE 10.0
#define ITERATIONS 3

#define FRONT 0
#define MIDDLE 1
#define BACK 2

@class Actor;
@class ActorInstance;
@class ActorType;
@class BehaviorManager;
@class CollisionListener;
@class GLESDebugDraw;
@class GroupDef;
@class Region;
@class Scene;
@class SHThumbstick;
@class SHThumbstickEvent;
@class TerrainRegion;
@class Transition;
@class FadeInTransition;


//----------------------------------------------------------------------------------
// Interface
//----------------------------------------------------------------------------------

@interface Game : SPStage <UIAccelerometerDelegate, GKLeaderboardViewControllerDelegate, GKAchievementViewControllerDelegate>
{	
	@private
        BOOL mPaused;
        BOOL mJustLoaded;
    
        int mWorldID;
        BOOL mAdVisible;
    
		float mScrollX;
		float mScrollY;
	
		BOOL mAnyStartTouch;
		BOOL mAnyMoveTouch;
		BOOL mAnyEndTouch;
	
		SPTouch* mStartTouch;
		SPTouch* mMoveTouch;
		SPTouch* mEndTouch;
	
		NSArray* mStartTouches;
		NSArray* mMoveTouches;
		NSArray* mEndTouches;
		int mTouchCount;
        int mTapCount;
	
		double mAccumulator;
    
        SPSprite* mStage;
        SPSprite* mStageHolder;
        //SPSprite* mHUDLayer;
    
        BOOL mAutoRotate;
	
		//Debug
		BOOL mDebugDrawing;
		GLESDebugDraw* mDebugDrawer;
	
		int mInitSceneID;
		int mGameID;
		int mGameName;
	
		//Transitioning
		int mEnterTimer;
		int mLeaveTimer;
		Transition* mLeave;
		Transition* mEnter;
		int mSceneToEnter;
	
		//Physics
		#ifdef __cplusplus
		b2World* mWorld;
		#endif
		CollisionListener* mCollisionListener;
	
		float mPhysicalWidth;
		float mPhysicalHeight;
		int mTop;
		int mLeft;
		int mBottom;
		int mRight;
	
		//Model
		NSMutableDictionary* mGameAttributes;
        NSMutableString* mDocPath;
    
		Scene* mScene;
		Actor* mCamera;
		BehaviorManager* mBehaviors;
		
		NSMutableDictionary* mGroups;
		NSMutableDictionary* mAllActors;
		int mNextID;
		
		NSMutableDictionary* mJoints;
		NSMutableDictionary* mRegions;
		NSMutableDictionary* mTerrainRegions;
		//skip region listeners
	
		NSMutableDictionary* mLayers;
		NSMutableDictionary* mTileLayers;
		NSMutableDictionary* mDynamicTiles;
		
		//List of Actors, Coincidentally the same as allActors at the moment
		//May be removed in the future since it's useless
		NSMutableDictionary* mActorsOnScreen;
		
		//Map<int, Group> //What is this used for??
		NSMutableDictionary* mActorsToRender;
        NSMutableDictionary* mHUDActorsToRender;
		
		NSMutableSet* mHudActors;
        NSMutableSet* mHudActorsToAdd;
        NSMutableSet* mHudActorsToRemove;
		
		//HashMap<Integer, HashSet<Actor>>
		NSMutableDictionary* mActorsOfType;
	
		NSMutableSet* mToAdd;
		NSMutableSet* mToRemove;
		
		//HashMap<Integer, HashSet<Actor>>
		NSMutableDictionary* mRecycledActorsOfType;
		
		//int[]
		//index -> order
		//value -> layerID
		NSMutableDictionary* mLayersToDraw;
		NSMutableDictionary* mLayerOrders;
		
		int mTopLayer;
		int mBottomLayer;
		int mMiddleLayer;
		
		//List<DeferredActor>
		NSMutableArray* mActorsToCreateInNextScene;
	
		UIAccelerometer* mAccelerometer;
		double mAccelX;
		double mAccelY;
		double mAccelZ;
        int mAccelFPS;
    
        float touchX;
        float touchY;
        float downX;
        float downY;
        float dragX;
        float dragY;
        float releaseX;
        float releaseY;
    
        SHThumbstick* mThumbstick;
        float mThumbstickDirection;
        float mThumbstickDistance;
        BOOL mShowingThumbstick;
    
        BOOL mSwipeUp;
        BOOL mSwipeDown;
        BOOL mSwipeLeft;
        BOOL mSwipeRight;
    
		Actor* mLastCreatedActor;
		Region* lastCreatedRegion;
		#ifdef __cplusplus
		b2Joint* lastCreatedJoint;
		#endif
		//joint, region
	
		float mElapsedTime;
	
		//Graphics
        NSMutableArray* mVertices;
        BOOL mPolyFill;
    
        BOOL mLocalSpace;
		int mDrawX;
		int mDrawY;
		float mDrawAlpha;
		NSString* mFontFace;
		float mFontSize;
		int mThickness;
		float mRed;
		float mGreen;
		float mBlue;
		float mStrokeRed;
		float mStrokeGreen;
		float mStrokeBlue;
        SPTextField* mTextHelper;
        SPRenderSupport* mCurrSupport;
    
        NSMutableDictionary* mStringWidths;
        NSMutableDictionary* mStringHeights;
    
        float mShakeIntensity;
        int mShakeTimer;
        BOOL mIsShaking;
	
		NSMutableSet* mSounds;
        NSMutableArray* mChannels;
	
		BOOL mLandscape;
        BOOL mGameCenterEnabled;
    
        BOOL miPad;
        BOOL miPhone;
    
        int mLargerDimension;
        int mSmallerDimension;
        int mSelfWidth;
        int mSelfHeight;
    
        UIWindow* mWindow;
        UIViewController* tempVC;
	
		//---
	
		//Temporary???
		NSMutableArray* mActors;
	
		ParallaxArea* mParallax;
		SPSprite* playfield;
	
		//Internal
        int mBarBorder;
        int mBarWidth;
        int mBarHeight;
        int mBarX;
        int mBarY;
        int mBarOffsetX;
        int mBarOffsetY;
        float mBarScale;
        int mBarOffsetXTablet;
        int mBarOffsetYTablet;
        float mBarScaleTablet;
    
        int mBarAlign;
    
        uint mBGColor;
        uint mBarColor;
        uint mBarBGColor;
        uint mBarBorderColor;
    
        double mProgress;
        double mTargetProgress;

        SPImage* mPreloaderImage;
        SPQuad* mProgressBar;
        SPQuad* mPreloaderBG;
        SPQuad* mBorderBG;
        SPQuad* mBarBackground;
    
        BOOL mStarted;
		int frameCount;
		double totalTime;
        int mFPS;
    
        //IAP's
        IAPHelper* mPurchases;
    
        //For auto-retaining and clearing
        NSMutableArray* mWebRequests;
        NSMutableArray* mWeakRefCache;
        NSMutableArray* mDeadCache;
        NSMutableArray* mMasterCache;
    
        NSArray* mMacAddresses;
        BOOL mShutdown;
    
		//We don't need tasks (juggler does it)
    
        float mMasterVolume;
    
    //Events
    NSMutableDictionary* mWhenKeyPressedListeners; //does this still apply for virtual key presses?
    NSMutableDictionary* mWhenTypeGroupCreatedListeners;
    NSMutableDictionary* mWhenTypeGroupDiesListeners;
    NSMutableDictionary* mWhenTypeGroupPositionListeners;
    NSMutableDictionary* mCollisionListeners;
    NSMutableDictionary* mSoundListeners;
    
    NSMutableArray* mWhenUpdatedListeners;
    NSMutableArray* mWhenDrawingListeners;
    NSMutableArray* mWhenTouchPressedListeners;
    NSMutableArray* mWhenTouchReleasedListeners;
    NSMutableArray* mWhenTouchMovedListeners;
    NSMutableArray* mWhenTouchDraggedListeners;
    NSMutableArray* mWhenPausedListeners;
    NSMutableArray* mWhenTiltedListeners;
    NSMutableArray* mWhenSwipedListeners;
}

+ (Game*) game;


//----------------------------------------------------------------------------------
// Methods
//----------------------------------------------------------------------------------

- (void)configurePreloader:(uint)bgColor barColor:(uint)barColor barBGColor:(uint)barBGColor barBorderColor:(uint)barBorderColor barBorder:(int)barBorder barWidth:(int)barWidth barHeight:(int)barHeight barX:(int)barX barY:(int)barY barOffsetX:(int)barOffsetX barOffsetY:(int)barOffsetY barScale:(float)barScale barOffsetXTablet:(int)barOffsetXTablet barOffsetYTablet:(int)barOffsetYTablet barScaleTablet:(float)barScaleTablet;

- (void)begin; //called just once at the start

//Orientation
- (BOOL)isLandscape;

//Scene Switching and Loading
- (Scene*)getSceneWithName:(NSString*)name;
- (void)cleanup; //for scene switching
- (void)switchScene:(int)sceneID leave:(Transition*)leave enter:(Transition*)enter;
- (void)enterScene;
- (void)leaveScene;
- (BOOL)isTransitioning;
- (void)loadScene:(int)sceneID;

- (void)loadDeferredActors;
- (void)loadForegrounds;
- (void)loadBackgrounds;
- (void)initActorScripts;
+ (void)initBehaviors:(BehaviorManager*)manager values:(NSMutableDictionary*)values parent:(NSObject*)parent initialize:(BOOL)initialize;
- (void)initPhysics;
- (void)enableGlobalSleeping;
- (void)disableGlobalSleeping;
- (void)loadCamera;
- (void)loadActors;
- (void)loadRegions;
- (void)loadTerrainRegions;
- (void)loadJoints;

//Actors
- (void)createActorInNextScene:(ActorType*)type x:(int)x y:(int)y layer:(int)layer;
- (Actor*)createActor:(ActorInstance*)ai offset:(BOOL)offset;
- (void)removeActor:(Actor*)a;
- (void)removeActorFromLayer:(Actor*)a layerID:(int)layerID;
- (void)moveActorToLayer:(Actor*)a layerID:(int)layerID;
- (void)recycleActor:(Actor*)a;
- (Actor*)getRecycledActorOfType:(ActorType*)type x:(int)x y:(int)y layerConst:(int)layerConst;
- (Actor*)createActorOfType:(ActorType*)type x:(int)x y:(int)y layerConst:(int)layerConst;

//BG's
- (void)setBackground:(int)backgroundID;

//Terrain
- (void)loadTerrain;
- (void)createDynamicTile; //TODO: This has a much longer signature
- (int)getTopLayer;
- (int)getBottomLayer;
- (int)getMiddleLayer;

//Updates
//Update is already part of superclass
- (void)postUpdate:(float)elapsedTime;
- (void)innerUpdate;
//No addTask needed
- (void)cameraFollow:(Actor*)actor lockX:(BOOL)lockX lockY:(BOOL)lockY;
- (void)addRemoveActors;
- (void)handleCollision:(Actor*)a event:(Collision*)event;

- (void)pause;
- (void)unpause;
- (BOOL)isPaused;

//Sound
- (void)soundFinished:(SoundChannel*)channel;
- (void)onSoundStarted:(SPSoundChannel*)ch;
- (void)onSoundCompleted:(SPEvent*)event;
- (void)stopAllSounds;

//Drawing and Layers
//Render is already part of superclass
- (void)moveToLayerOrder:(Actor*)a layerOrder:(int)layerOrder;
- (void)sendToBack:(Actor*)a;
- (void)sendBackward:(Actor*)a;
- (void)bringToFront:(Actor*)a;
- (void)bringForward:(Actor*)a;
- (int)getOrderForLayerID:(int)layerID;
- (int)getIDFromLayerOrder:(int)layerOrder;

//Physics
- (float)getPhysicalWidth;
- (float)getPhysicalHeight;
+ (float)toPhysicalUnits:(float)n;
+ (int)toPixelUnits:(float)n;
#ifdef __cplusplus
+ (b2Vec2)vToPhysicalUnits:(b2Vec2)v;
+ (b2Vec2)vToPixelUnits:(b2Vec2)v;
#endif
- (float)getGravityX;
- (float)getGravityY;
- (void)setGravity:(float)x y:(float)y;
//No debug draw

//Messaging
- (id)getAttribute:(NSString*)attName from:(NSString*)behaviorName;
- (void)setAttribute:(NSString*)attName from:(NSString*)behaviorName withValue:(void*)value;
- (id)say:(NSString*)msg to:(NSString*)recipient withArgs:(NSArray*)args;
- (void)shout:(NSString*)msg;

//Actors
- (Actor*)getActor:(int)ID;
- (NSMutableSet*)getActorsOfType:(ActorType*)type;
- (NSMutableSet*)getRecycledActorsOfType:(ActorType*)type;
- (void)addAlwaysOnActor:(Actor*)a;
- (void)addHUDActor:(Actor*)a alterLayer:(BOOL)alterLayer;
- (void)removeHUDActor:(Actor*)a;

//Groups
- (NSNumber*)getIDForGroup:(NSMutableSet*)group;
- (NSMutableSet*)getGroup:(int)ID actor:(Actor*)actor;
- (GroupDef*)getGroupName:(int)groupID actor:(Actor*)actor;


//Joints
- (int)nextJointID;

#ifdef __cplusplus
- (void)addJoint:(b2Joint*)j;
- (b2Joint*)getJoint:(int)ID;
- (void)destroyJoint:(b2Joint*)j;
- (b2DistanceJoint*)createStickJoint:(b2Body*)one two:(b2Body*)two jointID:(int)jointID collide:(BOOL)collide damping:(float)damping frequencyHz:(int)frequencyHz;
- (b2DistanceJoint*)createCustomStickJoint:(b2Body*)one x1:(float)x1 y1:(float)y1 two:(b2Body*)two x2:(float)x2 y2:(float)y2;
- (b2RevoluteJoint*)createHingeJoint:(b2Body*)one two:(b2Body*)two pt:(b2Vec2*)pt jointID:(int)jointID collide:(BOOL)collide limit:(BOOL)limit motor:(BOOL)motor lower:(float)lower upper:(float)upper torque:(float)torque speed:(float)speed;
- (b2LineJoint*)createSlidingJoint:(b2Body*)one two:(b2Body*)two dir:(b2Vec2*)dir jointID:(int)jointID collide:(BOOL)collide limit:(BOOL)limit motor:(BOOL)motor lower:(float)lower upper:(float)upper force:(float)force speed:(float)speed;
#endif

//Regions
#ifdef __cplusplus
- (Region*)createRegion:(float)x y:(float)y shape:(b2Shape*)shape offset:(BOOL)offset;
#endif
- (Region*)createBoxRegion:(float)x y:(float)y width:(float)width height:(float)height;
- (Region*)createCircularRegion:(float)x y:(float)y r:(float)r;
- (void)addRegion:(Region*)r;
- (void)removeRegion:(int)ID;
- (Region*)getRegion:(int)ID;
- (NSMutableDictionary*)getRegions;
- (int)nextRegionID;
- (BOOL)isInRegion:(Actor*)a r:(Region*)r;

//Terrain Regions
#ifdef __cplusplus
- (Region*)createTerrainRegion:(int)x y:(int)y shape:(b2Shape*)shape offset:(BOOL)offset;
#endif
- (TerrainRegion*)createBoxTerrainRegion:(int)x y:(int)y width:(int)width height:(int)height;
- (TerrainRegion*)createCircularTerrainRegion:(int)x y:(int)y r:(int)r;
- (void)addTerrainRegion:(TerrainRegion*)r;
- (void)removeTerrainRegion:(int)ID;
- (TerrainRegion*)getTerrainRegion:(int)ID;
- (NSMutableDictionary*)getTerrainRegions;
- (int)nextTerrainRegionID;

//Game Attributes
- (void)setGameAttribute:(NSString*)name value:(id)value;
- (id)getGameAttribute:(NSString*)name;
- (void)saveGame;
- (void)loadGame;

//On/Off Screen
- (void)setOffscreenTolerance:(int)top left:(int)left bottom:(int)bottom right:(int)right;
- (int)fetchActorsToRender;
- (void)cacheActors;
- (void)initLayers;

//User Input
- (int)numTouches;
- (int)numTaps;
- (BOOL)beganTouch;
- (BOOL)activeTouch;
- (BOOL)draggedTouch;
- (BOOL)releasedTouch;
- (int)getTouchX;
- (int)getTouchY;
- (int)getTouchDownX;
- (int)getTouchDownY;
- (int)getTouchDragX;
- (int)getTouchDragY;
- (int)getTouchReleaseX;
- (int)getTouchReleaseY;
- (void)setAccelFPS:(int)accelFPS;

- (void)enableThumbstick;
- (void)disableThumbstick;
- (void)onThumbstickTouch:(SHThumbstickEvent*)event;
- (void)onThumbstickMove:(SHThumbstickEvent*)event;
- (void)onThumbstickTouchUp:(SHThumbstickEvent*)event;
- (void)onThumbstickChanged:(SHThumbstickEvent*)event;
- (void)setThumbstickX:(int)x;
- (void)setThumbstickY:(int)y;

- (void)enableSwipeDetection;
- (void)disableSwipeDetection;
- (void)onSwipe:(SHSwipeEvent*)event;

//Graphics API
- (void)resetGraphics;
- (void)drawString:(id)s x:(int)x y:(int)y;
- (int)getWidthForString:(NSString*)font string:(NSString*)s;
- (int)getHeightForString:(NSString*)font string:(NSString*)s;
- (void)drawRoundRect:(int)x y:(int)y w:(int)w h:(int)h arcSize:(int)arcSize;
- (void)fillRoundRect:(int)x y:(int)y w:(int)w h:(int)h arcSize:(int)arcSize;
- (void)drawRect:(int)x y:(int)y w:(int)w h:(int)h;
- (void)fillRect:(int)x y:(int)y w:(int)w h:(int)h;
- (void)fillCircle:(int)xCenter y:(int)yCenter radius:(int)radius;
- (void)drawCircle:(int)xCenter y:(int)yCenter radius:(int)radius;
- (void)drawPixel:(int)x y:(int)y;
- (void)drawLine:(int)x1 y1:(int)y1 x2:(int)x2 y2:(int)y2;

- (void)startDrawPolygon;
- (void)startFillPolygon;
- (void)startPolygon:(BOOL)filled;
- (void)addPointToPolygon:(float)x y:(float)y;
- (void)endPolygon;

- (void)drawImageForActor:(Actor*)a;

- (void)setColor:(UIColor*)color;
- (void)setStrokeColor:(UIColor*)color;
- (void)setStrokeThickness:(int)thickness;
- (void)setAlpha:(float)alpha;
- (void)setFont:(NSString*)face size:(float)size;
//gradient
//blendmode

- (NSString*)getFont;

- (void)translate:(int)x y:(int)y;
- (void)moveTo:(int)x y:(int)y;

- (void)toScreenSpace;
- (void)toActorSpace;
- (BOOL)isInActorSpace;
- (BOOL)isInScreenSpace;

- (void)shakeScreen:(float)intensity forDuration:(int)duration;
- (void)stopShakingScreen;
//- (void)zoomTo:(float)pct overTime:(float)time;


//Web
- (void)retainWebResponse:(NSString*)data;
- (id)addToWeakCache:(id)data;

//Ads
- (void)showAd;
- (void)hideAd;

//Game Center
- (void)initGameCenter;
- (BOOL)isGameCenterAPIAvailable;
- (void)authenticateLocalPlayer;

- (void)reportScore:(int64_t)score forCategory:(NSString*)category;
- (void)showLeaderboard:(NSString*)category;

- (void)reportAchievement:(NSString*)name percentComplete:(float)percent;
- (void)showAchievements;
- (void)resetAchievements;

- (NSString*)getGameCenterName;
- (NSString*)getGameCenterID;
- (BOOL)isPlayerAuthenticated;
- (void)setWindow:(UIWindow*)window;

//IAPs
- (void)initPurchases:(NSSet*)products;
- (void)buyProduct:(NSString*)productID withCallback:(IAPTask)callback_;
- (BOOL)purchasedProduct:(NSString*)productID;
- (NSSet*)getProductIdentifiers;

//Preloader
- (void)setProgress:(double)percent;

- (void)setMacAddress:(NSString*)macAddress;


//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) int worldID;
@property (nonatomic, assign) int FPS;

@property (nonatomic, assign) Actor* camera;
@property (nonatomic, assign) float scrollX;
@property (nonatomic, assign) float scrollY;

@property (nonatomic, assign) int drawX;
@property (nonatomic, assign) int drawY;
@property (nonatomic, assign) float drawAlpha;

@property (nonatomic, assign) int left;
@property (nonatomic, assign) int right;
@property (nonatomic, assign) int top;
@property (nonatomic, assign) int bottom;

@property (nonatomic, assign) SPTouch* startTouch;
@property (nonatomic, assign) SPTouch* moveTouch;
@property (nonatomic, assign) SPTouch* endTouch;

@property (nonatomic, assign) NSArray* startTouches;
@property (nonatomic, assign) NSArray* dragTouches;
@property (nonatomic, assign) NSArray* endTouches;
@property (nonatomic, assign) NSArray* allTouches;

@property (nonatomic, assign) double accelX;
@property (nonatomic, assign) double accelY;
@property (nonatomic, assign) double accelZ;

@property (nonatomic, assign) SHThumbstick* thumbstick;
@property (nonatomic, assign) float thumbstickDirection;
@property (nonatomic, assign) float thumbstickDistance;

@property (nonatomic, assign) BOOL swipeUp;
@property (nonatomic, assign) BOOL swipeDown;
@property (nonatomic, assign) BOOL swipeLeft;
@property (nonatomic, assign) BOOL swipeRight;

@property (nonatomic, assign) BOOL iPhone;
@property (nonatomic, assign) BOOL iPad;

@property (nonatomic, assign) int largerDimension;
@property (nonatomic, assign) int smallerDimension;

#ifdef __cplusplus
@property (nonatomic, assign) b2World *world;
#endif

@property (nonatomic, assign) SPSprite* theStage;
@property (nonatomic, assign) SPSprite* realStage;

@property (nonatomic, assign) NSMutableArray* channels;

@property (nonatomic, assign) double progress;

//

@property (nonatomic, assign) Scene* scene;
@property (nonatomic, assign) BehaviorManager* behaviors;
@property (nonatomic, assign) NSMutableDictionary* regions;
@property (nonatomic, assign) NSMutableDictionary* layers;

@property (nonatomic, assign) Actor* lastCreatedActor;
@property (nonatomic, assign) Region* lastCreatedRegion;
#ifdef __cplusplus
@property (nonatomic, assign) b2Joint* lastCreatedJoint;
#endif

@property (nonatomic, assign) BOOL autoRotate;
@property (nonatomic, assign) BOOL landscape;
@property (nonatomic, assign) BOOL gameCenterEnabled;
@property (nonatomic, assign) BOOL debugDrawing;
@property (nonatomic, assign) float elapsedTime;

@property (copy) NSMutableString* docPath;

@property (nonatomic, assign) float masterVolume;
@property (nonatomic, assign) SPRenderSupport* currSupport;
@property (nonatomic, assign) IAPHelper* purchases;

//Events
@property (nonatomic, assign) NSMutableDictionary* whenKeyPressedListeners;
@property (nonatomic, assign) NSMutableDictionary* whenTypeGroupCreatedListeners;
@property (nonatomic, assign) NSMutableDictionary* whenTypeGroupDiesListeners;
@property (nonatomic, assign) NSMutableDictionary* whenTypeGroupPositionListeners;
@property (nonatomic, assign) NSMutableDictionary* collisionListeners;
@property (nonatomic, assign) NSMutableDictionary* soundListeners;

@property (nonatomic, assign) NSMutableArray* whenUpdatedListeners;
@property (nonatomic, assign) NSMutableArray* whenDrawingListeners;
@property (nonatomic, assign) NSMutableArray* whenTouchPressedListeners;
@property (nonatomic, assign) NSMutableArray* whenTouchReleasedListeners;
@property (nonatomic, assign) NSMutableArray* whenTouchMovedListeners;
@property (nonatomic, assign) NSMutableArray* whenTouchDraggedListeners;
@property (nonatomic, assign) NSMutableArray* whenPausedListeners;
@property (nonatomic, assign) NSMutableArray* whenTiltedListeners;
@property (nonatomic, assign) NSMutableArray* whenSwipedListeners;

@end
