/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>
#import "TBXML.h"

#define REGION_ID -2
#define PLAYER_ID 0
#define TERRAIN_ID 1
#define DOODAD_ID 2
#define ACTOR_ID 3

@interface GameModel : NSObject 
{
	@private
		int mWidth;
		int mHeight;
		int mActualWidth;
		int mActualHeight;
		int mScale;
		
		int mGameID;
		int mDefaultSceneID;
    
        int mJoystickStyle;
		
		NSMutableDictionary* mGroups;
		NSMutableArray* mCollisionGroups;
		NSMutableDictionary* mShapes;
		NSMutableDictionary* mGameAttributes;
		NSMutableDictionary* mScenes;
        NSMutableDictionary* mControls;
        NSMutableDictionary* mControlStates;
        NSMutableDictionary* mControlStatesLast;
    
        NSMutableSet* mAtlasesToLoad;
    
		BOOL mLandscape;
    
        NSString* mGameName;
}

+ (GameModel*)get;
- (GameModel*)init:(TBXMLElement*)xml;
- (void)readScenes:(int)sceneCount;
- (NSMutableDictionary*)readGameAttributes:(TBXMLElement*)list;
- (NSMutableDictionary*)readGroups:(TBXMLElement*)xml;
- (NSMutableArray*)readCollisionGroups:(TBXMLElement*)xml;
- (NSMutableDictionary*)readShapes:(TBXMLElement*)xml;
- (NSMutableDictionary*)readControls:(TBXMLElement*)xml;
- (NSMutableSet*)readAtlases:(TBXMLElement*)xml;

- (BOOL)isKeyDown:(NSString*)keyname;
- (BOOL)isKeyPressed:(NSString*)keyname;
- (BOOL)isKeyReleased:(NSString*)keyname;
- (void)updateKeys;


//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) int width;
@property (nonatomic, assign) int height;
@property (nonatomic, assign) int actualWidth;
@property (nonatomic, assign) int actualHeight;

@property (nonatomic, assign) int gameID;
@property (nonatomic, assign) int defaultSceneID;

@property (nonatomic, assign) int joystickStyle;

@property (nonatomic, assign) NSMutableDictionary* groups;
@property (nonatomic, assign) NSMutableArray* collisionGroups;
@property (nonatomic, assign) NSMutableDictionary* shapes;
@property (nonatomic, assign) NSMutableDictionary* gameAttributes;
@property (nonatomic, assign) NSMutableDictionary* scenes;
@property (nonatomic, assign) NSMutableDictionary* controls;
@property (nonatomic, assign) NSMutableDictionary* controlStates;

@property (nonatomic, assign) NSMutableSet* atlasesToLoad;

@property (nonatomic, assign) BOOL landscape;
@property (nonatomic, assign) NSString* gameName;

@end
