/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>

@class CXMLElement;
@class Background;
@class TileLayer;
@class ActorInstance;
@class RegionDef;
@class TerrainRegionDef;

struct b2Color;
typedef struct b2Color b2Color;

@interface Scene : NSObject 
{
	@private
		int mID;
		NSString* mName;
		
		int mSceneWidth;
		int mSceneHeight;
		
		int mTileWidth;
		int mTileHeight;
		
		double mGravityX;
		double mGravityY;
		
		#ifdef __cplusplus
		b2Color* mStartColor;
		b2Color* mEndColor;
		#endif
	
		NSMutableArray* mBgs;
		NSMutableArray* mFgs;
		
		NSMutableDictionary* mTerrain;
		NSMutableDictionary* mActors;
		NSMutableDictionary* mBehaviorValues;
		NSMutableArray* mWireframes;
		NSMutableDictionary* mJoints;
		NSMutableDictionary* mRegions;
		NSMutableDictionary* mTerrainRegions;
}


-(Scene*)loadFromXML:(int)sceneID name:(NSString*)name;

- (NSMutableDictionary*)readRegions:(NSArray*)list;
- (RegionDef*)readRegion:(CXMLElement*)e;

- (NSMutableDictionary*)readTerrainRegions:(NSArray*)list;
- (TerrainRegionDef*)readTerrainRegion:(CXMLElement*)e;

- (NSMutableDictionary*)readJoints:(NSArray*)list;

#ifdef __cplusplus
- (b2JointDef*)readJoint:(CXMLElement*)e;
#endif

- (NSMutableDictionary*)readLayers:(NSArray*)list rawLayers:(NSMutableDictionary*)rawLayers;
- (NSMutableDictionary*)readRawLayers:(NSFileHandle*)bytes numLayers:(int)numLayers;
- (TileLayer*)readRawLayer:(NSFileHandle*)bytes length:(int)length;

- (NSMutableArray*)readBackgrounds:(NSArray*)list;
- (NSMutableArray*)readWireframes:(NSArray*)list;

- (NSMutableDictionary*)readActors:(NSArray*)list;
- (ActorInstance*)readActorInstance:(CXMLElement*)xml;

-(int)readInt:(NSFileHandle*)bytes;
-(short)readShort:(NSFileHandle*)bytes;


//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) int ID;
@property (nonatomic, assign) NSString* name;

@property (nonatomic, assign) int sceneWidth;
@property (nonatomic, assign) int sceneHeight;

@property (nonatomic, assign) int tileWidth;
@property (nonatomic, assign) int tileHeight;

@property (nonatomic, assign) double gravityX;
@property (nonatomic, assign) double gravityY;

#ifdef __cplusplus
@property (nonatomic, assign) b2Color* startColor;
@property (nonatomic, assign) b2Color* endColor;
#endif

@property (nonatomic, assign) NSMutableArray* bgs;
@property (nonatomic, assign) NSMutableArray* fgs;

@property (nonatomic, assign) NSMutableDictionary* terrain;
@property (nonatomic, assign) NSMutableDictionary* actors;
@property (nonatomic, assign) NSMutableDictionary* behaviorValues;
@property (nonatomic, assign) NSMutableArray* wireframes;
@property (nonatomic, assign) NSMutableDictionary* joints;
@property (nonatomic, assign) NSMutableDictionary* regions;
@property (nonatomic, assign) NSMutableDictionary* terrainRegions;

@end
